import { Account, Connection, PublicKey, Transaction, TransactionConfirmationStatus, TransactionSignature } from '@solana/web3.js';
import { WideBits } from './layout';
import BN from 'bn.js';
import { Market, OpenOrders, Orderbook } from '@project-serum/serum';
import { Order } from '@project-serum/serum/lib/market';
import Wallet from '@project-serum/sol-wallet-adapter';
export declare const tokenToDecimals: {
    BTC: number;
    ETH: number;
    SOL: number;
    SRM: number;
    USDT: number;
    USDC: number;
    WUSDT: number;
};
export declare class MangoGroup {
    publicKey: PublicKey;
    accountFlags: WideBits;
    tokens: PublicKey[];
    vaults: PublicKey[];
    indexes: {
        lastUpdate: BN;
        borrow: number;
        deposit: number;
    };
    spotMarkets: PublicKey[];
    oracles: PublicKey[];
    signerNonce: BN;
    signerKey: PublicKey;
    dexProgramId: PublicKey;
    totalDeposits: number[];
    totalBorrows: number[];
    maintCollRatio: number;
    initCollRatio: number;
    srmVault: PublicKey;
    admin: PublicKey;
    borrowLimits: number[];
    mintDecimals: number[];
    oracleDecimals: number[];
    nativeSrm: number | null;
    constructor(publicKey: PublicKey, decoded: any, nativeSrm?: number);
    getPrices(connection: Connection): Promise<number[]>;
    getMarketIndex(spotMarket: Market): number;
    getTokenIndex(token: PublicKey): number;
    getBorrowRate(tokenIndex: number): number;
    getDepositRate(tokenIndex: number): number;
    getUiTotalDeposit(tokenIndex: number): number;
    getUiTotalBorrow(tokenIndex: number): number;
    getTokenSymbol(tokenIndex: number): string;
    getTokenDecimals(tokenIndex: number): number;
}
export declare class MarginAccount {
    publicKey: PublicKey;
    createTime: number;
    accountFlags: WideBits;
    mangoGroup: PublicKey;
    owner: PublicKey;
    deposits: number[];
    borrows: number[];
    beingLiquidated: boolean;
    openOrders: PublicKey[];
    openOrdersAccounts: (OpenOrders | undefined)[];
    constructor(publicKey: PublicKey, decoded: any);
    getNativeDeposit(mangoGroup: MangoGroup, tokenIndex: number): number;
    getNativeBorrow(mangoGroup: MangoGroup, tokenIndex: number): number;
    getUiDeposit(mangoGroup: MangoGroup, tokenIndex: number): number;
    getUiBorrow(mangoGroup: MangoGroup, tokenIndex: number): number;
    loadOpenOrders(connection: Connection, dexProgramId: PublicKey): Promise<(OpenOrders | undefined)[]>;
    toPrettyString(mangoGroup: MangoGroup, prices: number[]): string;
    computeValue(mangoGroup: MangoGroup, prices: number[]): number;
    getValue(connection: Connection, mangoGroup: MangoGroup): Promise<number>;
    getDeposits(mangoGroup: MangoGroup): number[];
    getAssets(mangoGroup: MangoGroup): number[];
    getLiabs(mangoGroup: MangoGroup): number[];
    getAssetsVal(mangoGroup: MangoGroup, prices: number[]): number;
    getLiabsVal(mangoGroup: MangoGroup, prices: number[]): number;
    getCollateralRatio(mangoGroup: MangoGroup, prices: number[]): number;
    cancelAllOrdersByMarket(connection: Connection, client: MangoClient, programId: PublicKey, mangoGroup: MangoGroup, market: Market, bids: Orderbook, asks: Orderbook, owner: Account): Promise<TransactionSignature[]>;
}
export declare class MangoSrmAccount {
    publicKey: PublicKey;
    accountFlags: WideBits;
    mangoGroup: PublicKey;
    amount: number;
    constructor(publicKey: PublicKey, decoded: any);
    getUiSrmAmount(): number;
}
export declare class MangoClient {
    confirmLevel: TransactionConfirmationStatus;
    constructor(confirmLevel?: TransactionConfirmationStatus);
    sendTransaction(connection: Connection, transaction: Transaction, payer: Account, additionalSigners: Account[], timeout?: number): Promise<TransactionSignature>;
    sendTransactionDeprecated(connection: Connection, transaction: Transaction, payer: Account, additionalSigners: Account[]): Promise<TransactionSignature>;
    initMangoGroup(connection: Connection, programId: PublicKey, payer: PublicKey): Promise<void>;
    initMarginAccount(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, owner: Account): Promise<PublicKey>;
    deposit(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, token: PublicKey, tokenAcc: PublicKey, quantity: number): Promise<TransactionSignature>;
    withdraw(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, token: PublicKey, tokenAcc: PublicKey, quantity: number): Promise<TransactionSignature>;
    borrow(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, token: PublicKey, quantity: number): Promise<TransactionSignature>;
    borrowAndWithdraw(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, wallet: Wallet | Account, token: PublicKey, tokenAcc: PublicKey, withdrawQuantity: number): Promise<TransactionSignature>;
    settleBorrow(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, token: PublicKey, quantity: number): Promise<TransactionSignature>;
    /**
     * Call SettleFunds on each market, then call SettleBorrow for each token in one transaction
     * @param connection
     * @param programId
     * @param mangoGroup
     * @param marginAccount
     * @param markets
     * @param owner
     */
    settleAll(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, markets: Market[], owner: Account): Promise<TransactionSignature | null>;
    liquidate(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, liqeeMarginAccount: MarginAccount, // liquidatee marginAccount
    liqor: Account, // liquidator
    tokenAccs: PublicKey[], depositQuantities: number[]): Promise<TransactionSignature>;
    forceCancelOrders(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, liqeeMarginAccount: MarginAccount, liqor: Account, spotMarket: Market, limit: number): Promise<TransactionSignature>;
    partialLiquidate(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, liqeeMarginAccount: MarginAccount, liqor: Account, liqorInTokenWallet: PublicKey, liqorOutTokenWallet: PublicKey, inTokenIndex: number, outTokenIndex: number, maxDeposit: number): Promise<TransactionSignature>;
    depositSrm(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, owner: Account, srmAccount: PublicKey, quantity: number, mangoSrmAccount?: PublicKey): Promise<PublicKey>;
    withdrawSrm(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, mangoSrmAccount: MangoSrmAccount, owner: Account, srmAccount: PublicKey, quantity: number): Promise<TransactionSignature>;
    placeOrder(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, spotMarket: Market, owner: Account, side: 'buy' | 'sell', price: number, size: number, orderType?: 'limit' | 'ioc' | 'postOnly', clientId?: BN, timeout?: number): Promise<TransactionSignature>;
    placeAndSettle(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, spotMarket: Market, owner: Account, side: 'buy' | 'sell', price: number, size: number, orderType?: 'limit' | 'ioc' | 'postOnly', clientId?: BN, timeout?: number): Promise<TransactionSignature>;
    settleFunds(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, spotMarket: Market): Promise<TransactionSignature>;
    cancelOrder(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, spotMarket: Market, order: Order): Promise<TransactionSignature>;
    addMarginAccountInfo(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, marginAccount: MarginAccount, owner: Account, info: string): Promise<TransactionSignature>;
    getMangoGroup(connection: Connection, mangoGroupPk: PublicKey, srmVaultPk?: PublicKey): Promise<MangoGroup>;
    getMarginAccount(connection: Connection, marginAccountPk: PublicKey, dexProgramId: PublicKey): Promise<MarginAccount>;
    getAllMarginAccounts(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, filters?: [any]): Promise<MarginAccount[]>;
    getAllMarginAccountsWithBorrows(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup): Promise<MarginAccount[]>;
    getMarginAccountsForOwner(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, owner: Account | Wallet): Promise<MarginAccount[]>;
    getMangoSrmAccount(connection: Connection, mangoSrmAccountPk: PublicKey): Promise<MangoSrmAccount>;
    getMangoSrmAccountsForOwner(connection: Connection, programId: PublicKey, mangoGroup: MangoGroup, owner: Account | Wallet): Promise<MangoSrmAccount[]>;
}
//# sourceMappingURL=client.d.ts.map